import psutil
import datetime
import sqlite3
from telegram import Bot, Update
from telegram.ext import Updater, CommandHandler

bot_token = "把这里的替换为你的电报机器人的token值"

monthly_limit = 2000 * 1024 * 1024 * 1024  # 2000GB 这是套餐流量额度
flow_correction = -279.581  # 附加校正参数

def get_bandwidth_usage():
    net_io = psutil.net_io_counters()
    total_out = net_io.bytes_sent
    total_in = net_io.bytes_recv
    total_usage = total_out + total_in + flow_correction * 1024 * 1024 * 1024
    used_flow = convert_bytes(total_usage)
    total_limit = "2000GB"  # 每月总带宽限制
    return f"本月带宽使用情况: {used_flow}/{total_limit}"

def convert_bytes(bytes):
    sizes = ["B", "KB", "MB", "GB", "TB"]
    i = 0
    while bytes >= 1024 and i < len(sizes) - 1:
        bytes /= 1024
        i += 1
    return f"{bytes:.3f} {sizes[i]}"

def get_server_status():
    now = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
    uptime = datetime.datetime.fromtimestamp(psutil.boot_time()).strftime("%Y-%m-%d %H:%M:%S")
    runtime = datetime.datetime.strptime(now, "%Y-%m-%d %H:%M:%S") - datetime.datetime.strptime(uptime, "%Y-%m-%d %H:%M:%S")
    cpu_usage = psutil.cpu_percent()
    memory_usage = psutil.virtual_memory().percent
    disk_usage = psutil.disk_usage('/').percent
    bandwidth_usage = get_bandwidth_usage()
    return f"●瓦工 日本 软银 110.9刀机\n当前时间：{now}\n系统本次运行时长：{runtime}\n系统状态：CPU {cpu_usage:.2f}% 内存 {memory_usage:.2f}% 磁盘 {disk_usage:.2f}%\n{bandwidth_usage}\n每月流量重置日期：28号"

def start_command(update: Update, context):
    chat_id = update.message.chat_id
    server_status = get_server_status()
    context.bot.send_message(chat_id=chat_id, text=server_status)

def send_database_data(update: Update, context):
    chat_id = update.message.chat_id
    port_number = context.args[0]  # 获取用户输入的端口号

    conn = sqlite3.connect('/etc/x-ui/x-ui.db')
    cursor = conn.cursor()
    cursor.execute("SELECT * FROM inbounds")
    rows = cursor.fetchall()

    data_output = ""
    for row in rows:
        if str(row[12]) == port_number:  # 检查端口号是否与用户输入匹配
            data = {
                "ID": row[0],
                "端口号": row[12],  # 调整索引位置以匹配实际字段位置
                "上传流量": round(row[2] / (1024 * 1024 * 1024), 4),
                "下载流量": round(row[3] / (1024 * 1024 * 1024), 4),
                "已使用流量": round((row[2] + row[3]) / (1024 * 1024 * 1024), 4),
                "流量限额": round(row[4] / (1024 * 1024 * 1024), 4)
            }
            output = "ID: {ID}, 端口: {端口号}, 上传: {上传流量}GB, 下载: {下载流量}GB, 已使用: {已使用流量}GB, 限额: {流量限额}GB"
            data_output += output.format(**data) + "\n"

    conn.close()    
    if data_output == "":
        data_output = "没有找到与该端口号相关的数据。"

    context.bot.send_message(chat_id=chat_id, text=data_output)

def get_traffic_data(update: Update, context):
    chat_id = update.message.chat_id

    conn = sqlite3.connect('/etc/x-ui/x-ui.db')
    cursor = conn.cursor()
    cursor.execute("SELECT * FROM inbounds")
    rows = cursor.fetchall()

    data_output = ""
    for row in rows:
        data = {
            "ID": row[0],
            "端口号": row[12],  # 调整索引位置以匹配实际字段位置
            "上传流量": round(row[2] / (1024 * 1024 * 1024), 4),
            "下载流量": round(row[3] / (1024 * 1024 * 1024), 4),
            "已使用流量": round((row[2] + row[3]) / (1024 * 1024 * 1024), 4),
            "流量限额": round(row[4] / (1024 * 1024 * 1024), 4)
        }
        output = "ID: {ID}, 端口: {端口号}, 上传: {上传流量}GB, 下载: {下载流量}GB, 已使用: {已使用流量}GB, 限额: {流量限额}GB"
        data_output += output.format(**data) + "\n"

    conn.close()
    if data_output == "":
        data_output = "没有找到与该端口号相关的数据。"

    context.bot.send_message(chat_id=chat_id, text=data_output)

# 创建一个Updater对象
updater = Updater(token=bot_token, use_context=True)
dispatcher = updater.dispatcher

# 添加一个命令处理程序来处理 /start 命令
start_handler = CommandHandler('start', start_command)
dispatcher.add_handler(start_handler)

# 添加一个命令处理程序来处理 /TrafficCheck 命令
traffic_handler = CommandHandler('TrafficCheck', get_traffic_data)
dispatcher.add_handler(traffic_handler)

# 添加一个命令处理程序来处理 /GetFlowData 命令
flow_data_handler = CommandHandler('GetFlowData', send_database_data)
dispatcher.add_handler(flow_data_handler)

# 启动Bot
updater.start_polling()